//Functions for managing the main application cycle, registering the callbacks
 // to create an application and its completion
#include <app.h>

// Functions for displaying messages in the logging window
#include <dlog.h>

/// Functions for creating and managing user interface elements. Here are all the widgets,
// such as: windows, buttons, etc.
#include <Elementary.h>

//Functions for using callback for hardware Back button and for
// using extended widgets for a circle screen
#include <efl_extension.h>

static const int FIRST_ITEM  = 1;
static const int SECOND_ITEM = 2;

typedef struct _UIData {
   Evas_Object *win;
   Evas_Object *conform;
   Evas_Object *rotary_selector;

   char *image_path;
} UIData;

static void
_win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
   UIData *ui = data;

   elm_win_lower(ui->win);
}

static bool
_app_control_extra_foreach_data_cb(app_control_h app_control,
                                   const char *key,
                                   void *user_data)
{
   UIData *ui   = (UIData *)user_data;
   char **value = NULL;
   int length   = 0;

   app_control_get_extra_data_array(app_control, key, &value, &length);
   if (length > 0)
     {
        if (ui->image_path)
          free(ui->image_path);

        ui->image_path = strdup(value[0]);
        dlog_print(DLOG_INFO, LOG_TAG, "Path to the image: %s", ui->image_path);

        for (int i = 0; i < length; ++i)
          {
             free(value[i]);
          }

        free(value);
     }

   return false;
}

static void
_pick_image_cb(app_control_h request,
               app_control_h reply,
               app_control_result_e result,
               void *user_data)
{
   app_control_foreach_extra_data(reply, _app_control_extra_foreach_data_cb, user_data);
}

static void
_item_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
   dlog_print(DLOG_INFO, "lesson27", "Element is pressed");

   app_control_h a_control;
   UIData *ui                    = (UIData *)data;
   Eext_Object_Item *item = eext_rotary_selector_selected_item_get(ui->rotary_selector);
   int *item_data                = item->data;
   bool is_first_item            = (*item_data == FIRST_ITEM);
   const char *type_operation    = is_first_item ? APP_CONTROL_OPERATION_PICK : APP_CONTROL_OPERATION_VIEW;
   app_control_reply_cb reply_cb = is_first_item ? _pick_image_cb : NULL;

   app_control_create(&a_control);
   app_control_set_operation(a_control, type_operation);
   if (!is_first_item)
     app_control_set_uri(a_control, ui->image_path);

   app_control_set_mime(a_control, "image/png");
   if (app_control_send_launch_request(a_control, reply_cb, data) != APP_CONTROL_ERROR_NONE)
     dlog_print(DLOG_INFO, "lesson27", "Application running error");

   app_control_destroy(a_control);
}

static void
_rotary_selector_create(UIData *ui)
{
   ui->rotary_selector = eext_rotary_selector_add(ui->conform);
   eext_rotary_object_event_activated_set(ui->rotary_selector, EINA_TRUE);

   Eext_Object_Item *item = eext_rotary_selector_item_append(ui->rotary_selector);
   item->data = (void *)&FIRST_ITEM;
   eext_rotary_selector_item_part_text_set(item,
                                           "selector,main_text",
                                           "Select");

   item = eext_rotary_selector_item_append(ui->rotary_selector);
   item->data = (void *)&SECOND_ITEM;
   eext_rotary_selector_item_part_text_set(item,
                                           "selector,main_text",
                                           "View");

   evas_object_smart_callback_add(ui->rotary_selector,
                                  "item,clicked",
                                  _item_clicked_cb,
                                  ui);

   elm_object_content_set(ui->conform, ui->rotary_selector);
   evas_object_show(ui->rotary_selector);
}

static void
_conformant_create(UIData *ui)
{
   ui->conform = elm_conformant_add(ui->win);

   //Specify to conformant occupies all allocated to him space
   // (horizontally and vertically)
   evas_object_size_hint_weight_set(ui->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

   _rotary_selector_create(ui);

   evas_object_show(ui->conform);
}

static void
_window_create(UIData *ui)
{
	// Create and configure the main window
   ui->win = elm_win_util_standard_add(NULL, NULL);

   // Create a conformant - in this case, the main container in the application,
   // which is also the interface between the application and
   // system elements, such as a keyboard.
   _conformant_create(ui);

   //  Set the size of the conformant widget the same as the size of the window
   elm_win_resize_object_add(ui->win, ui->conform);

   //Register the function that will be called
   //when you press the hardware Back button
   eext_object_event_callback_add(ui->win, EEXT_CALLBACK_BACK, _win_back_cb, ui);

   // Always display the window only after the entire base
     // user interface will be displayed.
   evas_object_show(ui->win);
}

static bool
_app_create_cb(void *data)
{
   UIData *ui = data;

   srand(time(NULL));

   _window_create(ui);

   return true;
}

static void
_app_terminate_cb(void *data)
{
   UIData *ui = data;

   //Before closing the application, delete the main widget (window),
  // then all  "children" widgets of this window (all widgets of this application) will be deleted.
   evas_object_del(ui->win);
}

int
main(int argc, char *argv[])
{
   UIData ui = {0,};
   ui_app_lifecycle_callback_s lifecycle_callbacks = {0,};

   //Register created functions to transfer them to the main loop of
   // this application.
   lifecycle_callbacks.create    = _app_create_cb;
   lifecycle_callbacks.terminate = _app_terminate_cb;

   // Calling the next function, launch the main application cycle
   return ui_app_main(argc, argv, &lifecycle_callbacks, &ui);
}
